/*
 *  Copyright (C) 2002  Harald Barth, Stockholm, Sweden.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 */
#include "consist.h"
RCSID("$Id: consist.c,v 1.3 2002/01/01 13:00:44 haba Exp $");
#include <stdio.h>
#include <stdlib.h>
#include "port.h"

extern char *optarg;
extern int optind, opterr, optopt;

int commandPortNum = 12345;
int infoPortNum = 12347;
char *serverName = "localhost";
int infoPort = -1;
int commandPort = -1;

int main( int argc, char* argv[] )
{
  char infomsg[MAXMSGLEN];
  char c;
  int i;
  int locosInArg = 0;

  
  while ((c=getopt(argc, argv, "+p:i:h")) != EOF) {
    switch(c) {
    case 'p': 
      { 
	int port;
	if (sscanf(optarg, "%d", &port) > 0)
	  commandPortNum = port;
      }
      break;
    case 'i': 
      { 
	int port;
	if (sscanf(optarg, "%d", &port) > 0)
	  infoPortNum = port;
      }
      break;
    case 'h':
    case '?':
      fprintf(stderr,"Usage: %s [-p commandport] [-i infoport] \\\n", argv[0]);
      fprintf(stderr,"                   <masterloco> <slave1> <slave2> ...\n\n");
      fprintf(stderr,"loco spec is '<proto> <addr>'\n");
      fprintf(stderr,"Example: %s 'M1 16' 'M1 60'\n\n", argv[0]);
      exit(255);
      break;
    }
  }
  
  for (i = optind; i < argc; i++) {
    char *protocol = NULL;
    int addr = -1;
    int n = 0;

    if ((protocol = (char *)malloc(strlen(argv[i])+1)) != NULL) {
      n = sscanf(argv[i],"%s %d",protocol,&addr);
      if (n == 2) {
	if (locosInArg == 0) 
	  locoAddMaster(protocol, addr);
	else
	  locoAddSlave(protocol,addr);
	locosInArg++;
	free(protocol);
      }
    }
  }

  if (locosInArg < 2) {
    fprintf(stderr,"At least one master and one slave is required\n");
    exit(255);
  }

  if ((infoPort = portOpen(serverName, infoPortNum)) == -1) {
    fprintf(stderr, "Info Port open failed\n");
    exit(255);
  }
  if ((commandPort = portOpen(serverName, commandPortNum)) == -1) {
    fprintf(stderr, "Info Port open failed\n");
    exit(255);
  }

  while(1) {
    portReadLine(infoPort, infomsg, MAXMSGLEN);
    srcp_setbuffer(infomsg);
    srcp_parse(commandPort);
  }
}
