/*
 *  Copyright (C) 2002  Harald Barth, Stockholm, Sweden.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 */
#include "consist.h"
RCSID("$Id: handler.c,v 1.2 2002/01/01 14:51:29 haba Exp $");

#include <stdio.h>
#include "loco.h"
#include "port.h"

void handleINFO(int socket, char *protocol, int addr, int direction,
		int v, int vmax, int func, int nro_f, int f[8]){
  int i;
  int remember = 0;
  static Loco *master = NULL;
  static int oldMasterDirection = 2;
  Loco *slave = NULL;
  Loco *loco = NULL;
  char slavemessage[MAXMSGLEN];

#ifdef DEBUG
  fprintf(stderr, "INFO GL %s %d %d %d %d %d %d ",
	  protocol, addr, direction, v, vmax, func, nro_f);
  for (i=0; i<nro_f; i++)  fprintf(stderr, "%d ", f[i]);
  fprintf(stderr,"\n");
#endif

  if ((loco = locoGet(protocol, addr)) != NULL){
    if (locoIsMaster(loco)) {
      /* Found a MASTER */
      master = loco;
      locoAddAllSlavesToMaster(master);
      
      if (master && master->direction != 2)
	oldMasterDirection = master->direction;
      
      locoUpdate(master, direction, 
		 v, vmax, func, nro_f, f);
      
      /* update all slaves to follow the master */
      for (slave = master->next; slave; slave = slave->next) {
	
	if (!slave->valid) {
#ifdef DEBUG
	  fprintf(stderr,"Skipping slave %d, not valid\n",slave->addr);
#endif
	  continue;
	}
	
	/* Direction */
	if (oldMasterDirection != 2 
	    && master->direction != 2
	    && oldMasterDirection != master->direction) {
	  /* the master turned, turn the slaves  */
	  if (slave->direction == 1) slave->direction = 0;
	  else if (slave->direction == 0) slave->direction = 1;
	}
	
	/* Speed */
	slave->v = (master->v * slave->v_max)/ master->v_max;
	
	locoComposeMessage(slavemessage, MAXMSGLEN, slave);
#ifdef DEBUG      
	fprintf(stderr, "%s", slavemessage);
#endif
	portWrite(socket, slavemessage);
      }
      
    } else {
      /* Found a SLAVE */
      slave = loco;
      locoUpdate(slave, direction, 
		 v, vmax, func, nro_f, f);
      if (master)
	locoAddSlaveToMaster(master, slave);
    }
  }
}
