/*
 *  Copyright (C) 2002  Harald Barth, Stockholm, Sweden.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 */
#include "consist.h"
RCSID("$Id: loco.c,v 1.2 2002/01/01 14:51:29 haba Exp $");

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "loco.h"

#define MAXNUMLOCOS 16
Loco *allLocos[MAXNUMLOCOS];

int locoUpdate(Loco *loco, int direction, 
	       int v, int vmax, int func, int nro_f, int *f){
  int i;
  
  if (loco == NULL) return 0;
  loco->direction = direction;
  loco->v = v;
  loco->v_max = vmax;
  loco->func = func;
  loco->nro_f = nro_f;
  for (i=0; i < loco->nro_f && i < 8; i++) 
    loco->f[i] = f[i];
  loco->valid = 1;
  return 1;
}

static 
Loco *newLoco(char *protocol, int addr){
  Loco *loco;
  int i;
  
  if ((loco = (Loco *)malloc(sizeof(Loco))) == NULL)
    return NULL;
  /* set fields that will not be changed later */
  loco->protocol = strdup(protocol);
  loco->addr = addr;
  /* for slaves later */
  loco->next = NULL;
  loco->enslaved = 0;
  loco->master = 0;
  loco->valid = 0;
  /* clear fields later set by upadateLoco */
  loco->direction = 0;
  loco->v = 0;
  loco->v_max = 0;
  loco->func = 0;
  for (i=0; i < 8; i++) 
    loco->f[i] = 0;
  return loco;
}

static 
Loco *locoDbFind(char *protocol, int addr){
  int i;

  for(i = 0; i < MAXNUMLOCOS && allLocos[i]; i++) {
    if (addr == allLocos[i]->addr &&
	strcmp(protocol,allLocos[i]->protocol) == 0)
      return allLocos[i];
  }
  return NULL;
}

static
int locoDbAdd(Loco *loco){
  int i;
  
  for(i = 0; i < MAXNUMLOCOS && allLocos[i]; i++);
  /* do nothing */
  if (i < MAXNUMLOCOS) {
    allLocos[i] = loco;
    return 1;
  }
  return 0;
}

static
Loco *locoAdd(char *protocol, int addr){
  Loco *loco;
  
  if ((loco = locoDbFind(protocol, addr)) == NULL) {
    /* not exist in hash */
    loco = newLoco(protocol, addr);
    if (loco == NULL) 
      return NULL;
    if (locoDbAdd(loco) == 0)
      return NULL;
  }
  return loco;
}

void locoComposeMessage(char *locomessage, int msglen, Loco *loco){

  int printed = 0;
  int left = msglen;
  int i = 0;
  
  char *s = locomessage;
  printed = snprintf(s, left, "SET GL %s %d %d %d %d %d %d\n",
		     loco->protocol, loco->addr, 
		     loco->direction, loco->v, loco->v_max,
		     loco->func, loco->nro_f);
  for (i = 0; i < loco->nro_f && i < 8; i++) {
    if (printed + 1 >= left) 
      return;
    s += printed - 1;
    left -= printed - 1;
    printed = snprintf(s, left, " %d\n", loco->f[i]);
  }
}

Loco *locoGet(char *protocol, int addr){
  return locoDbFind(protocol, addr);
}

int locoIsMaster(Loco *loco){
  return (loco && loco->master);
}

Loco *locoAddMaster(char *protocol, int addr) {
  Loco *loco;
  
  if ((loco = locoAdd(protocol, addr)) != NULL) {
    loco->master = 1;
    return loco;
  }
  return NULL;
}

Loco *locoAddSlave(char *protocol, int addr){
  return locoAdd(protocol, addr);
}

void locoAddSlaveToMaster(Loco *master, Loco *slave){
  if (slave == NULL) return;
  if (slave->enslaved) return;
  if (master == NULL) return;
  if (master->next == NULL) {
    slave->enslaved = 1;
    master->next = slave;
    return;
  }
  locoAddSlaveToMaster(master->next, slave);
}

void locoAddAllSlavesToMaster(Loco *master){
  int i;
  
  if (master == NULL) return;
  if (master->next != NULL) return;
  
  for(i = 0; i < MAXNUMLOCOS && allLocos[i]; i++)
    if (!locoIsMaster(allLocos[i]))
      locoAddSlaveToMaster(master, allLocos[i]);
}


